--[[	HideBlizzard	]]

local pairs = pairs
local string = string
local table = table
local tonumber = tonumber
local type = type

----------------------

local InCombatLockdown = _G.InCombatLockdown

----------------------

local addonName, ns = ...
local L = ns.L
HideBlizzard = LibStub("AceAddon-3.0"):NewAddon(addonName)
local addon = HideBlizzard
addon:SetEnabledState(true)
addon:SetDefaultModuleState(false)

----------------------

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")

----------------------

local ADDON_VERSION = GetAddOnMetadata(addonName, "Version")

----------------------

local moduleOptions = {}
local options = nil

local settings
local defaults = {
	profile = {
		modules = {
			["*"] = false,
		},
	},
}

----------------------

local function out(msg)

	DEFAULT_CHAT_FRAME:AddMessage("|cffAEAEE0"..addonName..":|r "..msg)

end

local function debug(msg)

	local debug = false

	if ( debug ) then

		out(msg)

	end

end

----------------------

local function getOptions()

	debug("getOptions")

	if ( not options ) then

		options = {
			name = "HideBlizzard",
			type = "group",
			childGroups = "tab",
			args = {
				header = {
					name = "|cffFFFF55"..L["HideBlizzard does nothing by default. Enable or Disable the selected modules."].."|r",
					order = 1,
					fontSize = "medium",
					width="full",
					type = "description",
				},
			},
		}
		for k,v in pairs(moduleOptions) do

			options.args[k] = type(v) == "function" and v() or v

		end

	end

	return options

end

----------------------

function addon:OnInitialize()

	debug("OnInitialize")

	addon.db = LibStub("AceDB-3.0"):New("HideBlizzardDB", defaults)

	settings = addon.db.profile

	addon:Updater()

	SlashCmdList[addonName] = function()

		debug("SlashCmdList")

		addon:openOptions()

	end

	SLASH_HideBlizzard1 = "/hideblizz"

end

----------------------

function addon:OnEnable()

	debug("OnEnable")

	AceConfigRegistry:RegisterOptionsTable(addonName, getOptions)

	addon:UpdateModules()

end

----------------------

function addon:OnDisable()

	debug("OnDisable")

end

----------------------

function addon:GetModuleEnabled(module)

	debug("GetModuleEnabled")

	return settings.modules[module]

end

----------------------

function addon:SetModuleEnabled(module, value)

	debug("SetModuleEnabled")

	local old = settings.modules[module]

	settings.modules[module] = value

	if ( old ~= value ) then

		if ( value ) then

			addon:EnableModule(module)

		else

			addon:DisableModule(module)

		end

	end

end

----------------------

function addon:RegisterModuleOptions(name, optionTbl, displayName)

	debug("RegisterModuleOptions")

	moduleOptions[name] = optionTbl

end

----------------------

function addon:UpdateModules()

	debug("UpdateModules")

	for k, v in self:IterateModules() do

		if ( self:GetModuleEnabled(k) and not v:IsEnabled() ) then

			self:EnableModule(k)

		elseif ( not self:GetModuleEnabled(k) and v:IsEnabled() ) then

			self:DisableModule(k)

		end

	end

end

----------------------

function addon:Updater()

	debug("Updater")

	if ( not HideBlizzardDB.version ) then

		HideBlizzardDB.version = tonumber(ADDON_VERSION)

		out("|cffE0E0AE"..L["Welcome! Type /hideblizz to configure."].."|r")

	elseif ( HideBlizzardDB.version ~= tonumber(ADDON_VERSION) ) then

		HideBlizzardDB.version = tonumber(ADDON_VERSION)

		out(string.format("|cff66FF66"..L["Updated to latest version"].."|r".." |cff66FF66( v%.1f )", HideBlizzardDB.version))

	end

end

----------------------

function addon:openOptions()

	debug("openOptions")

	if ( not InCombatLockdown() ) then

		AceConfigDialog:SetDefaultSize("HideBlizzard", 500, 550)
		AceConfigDialog:Open("HideBlizzard")

	else

		out("|cffFF6666"..L["Cannot open options while in combat."].."|r")

	end

end
