--[[	Pet	]]--

local _G = _G

----------------------

local GetCVarBool = _G.GetCVarBool
local GetNumSubgroupMembers = _G.GetNumSubgroupMembers
local UnitExists = _G.UnitExists

----------------------

local addonName, ns = ...
local L = ns.L
local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon(addonName)
Pet = HideBlizzard:NewModule("Pet")
local mod = Pet

----------------------

local dummy = function() end

local settings

local defaults = {
	profile = {
		["petactionbar"] = nil,
		["petcastbar"] = nil,
		["partypet"] = nil,
		["petunitframe"] = nil,
	},
}

----------------------

local function out(msg)

	DEFAULT_CHAT_FRAME:AddMessage("|cffAEAEE0"..addonName..":|r "..msg)

end

local function debug(msg)

	local debug = false

	if ( debug ) then

		out(msg)

	end

end

----------------------

function mod:getOptions()

debug("Pet getOptions")

return {
	name = L["Pet"],
	type = "group",
	arg = "Pet",
	get = function(info)

		local key = info[#info]
		local v = settings[key]

		return v

	end,
	set = function(info, value)

		local key = info[#info]

		settings[key] = value

	end,
	args = {
		enabled = {
			name = "|cffFFFF55"..L["Enable Pet"].."|r",
			desc = "|cffFFFF55"..L["Hides pet related frames."].."|r",
			descStyle = "inline",
			type = "toggle",
			order = 1,
			width = "full",
			get = function() return HideBlizzard:GetModuleEnabled("Pet") end,
			set = function(info, value) HideBlizzard:SetModuleEnabled("Pet", value) end,
		},
		spacer = {
			name = "",
			type = "description",
			order = 1.5,
		},
		petactionbar = {
			name = L["Pet Action Bar"],
			desc = L["Hides the pet action bar."],
			type = "toggle",
			order = 2,
			disabled = function() return not Pet:IsEnabled() end,
			set = function(info, v)

				settings.petactionbar = v

				mod:UpdatePetActionBar()

			end,
			
		},
		petcastbar = {
			name = L["Pet Cast Bar"],
			desc = L["Hides the pet casting bar."],
			type = "toggle",
			order = 3,
			disabled = function() return not Pet:IsEnabled() end,
			set = function(info, v)

				settings.petcastbar = v

				mod:UpdatePetCastBar()

			end,
		},
		partypet = {
			name = L["Party Pet Unit Frame"],
			desc = L["Hides the party pet unit frame."],
			type = "toggle",
			order = 4,
			disabled = function() return not Pet:IsEnabled() end,
			set = function(info, v)

				settings.partypet = v

				mod:UpdatePartyPet()

			end,
		},
		petunitframe = {
			name = L["Pet Unit Frame"],
			desc = L["Hides the pet unit frame."],
			type = "toggle",
			order = 5,
			disabled = function() return not Pet:IsEnabled() end,
			set = function(info, v)

				settings.petunitframe = v

				mod:UpdatePetUnitFrame()

			end,
		},
	},
}
end

----------------------

function mod:OnInitialize()

	debug("Pet OnInitialize")

	mod:SetEnabledState(HideBlizzard:GetModuleEnabled("Pet"))
	
	mod.db = HideBlizzard.db:RegisterNamespace("Pet", defaults)
	
	settings = mod.db.profile

	local options = mod:getOptions()

	HideBlizzard:RegisterModuleOptions("Pet", options, "Pet")

end

----------------------

function mod:OnEnable()

	debug("Pet OnEnable")

	mod:UpdateView()

end

----------------------

function mod:OnDisable()

	debug("Pet OnDisable")

	mod:UpdateView()

end

----------------------

function mod:UpdateView()

	debug("Pet UpdateView")

	mod:UpdatePetActionBar()
	mod:UpdatePetCastBar()
	mod:UpdatePartyPet()
	mod:UpdatePetUnitFrame()

end

----------------------

function mod:UpdatePetActionBar()

	if ( settings.petactionbar == nil ) then return end

	debug("UpdatePetActionBar")

	local PetActionBarFrame = _G.PetActionBarFrame

	if ( settings.petactionbar and mod:IsEnabled() ) then

		if ( UnitExists("pet") ) then

			PetActionBarFrame:Hide()
			PetActionBarFrame.Show = dummy

		end

	else

		PetActionBarFrame.Show = nil

		if ( UnitExists("pet") ) then

			PetActionBarFrame:Show()

		end

		settings.petactionbar = nil

	end

end

----------------------

function mod:UpdatePetCastBar()

	if ( settings.petcastbar == nil ) then return end

	debug("UpdatePetCastBar")

	local PetCastingBarFrame = _G.PetCastingBarFrame

	if ( settings.petcastbar and mod:IsEnabled() ) then

		if ( UnitExists("pet") ) then

			PetCastingBarFrame:Hide()
			PetCastingBarFrame.Show = dummy

		end

	else

		PetCastingBarFrame.Show = nil

		settings.petcastbar = nil

	end

end

----------------------

function mod:UpdatePartyPet()

	if ( settings.partypet == nil ) then return end

	debug("UpdatePartyPet")

	local isCompact = GetCVarBool("useCompactPartyFrames")

	if ( settings.partypet and mod:IsEnabled() ) then

		for i=1, MAX_PARTY_MEMBERS do

			if ( GetNumSubgroupMembers(i) and not isCompact ) then

				local pp = _G["PartyMemberFrame"..i.."PetFrame"]

				pp:Hide()
				pp.Show = dummy

			else

				local pp = _G["PartyMemberFrame"..i.."PetFrame"]

				pp:Hide()
				pp.Show = dummy

			end

		end

	else

		for i=1, MAX_PARTY_MEMBERS do

			if ( GetNumSubgroupMembers(i) and not isCompact ) then

				local pp = _G["PartyMemberFrame"..i.."PetFrame"]

				pp.Show = nil

				if (UnitExists("partypet"..i)) then

					pp:Show()

				end

			else

				local pp = _G["PartyMemberFrame"..i.."PetFrame"]

				pp.Show = nil

				if ( UnitExists("partypet"..i) and not isCompact ) then

					pp:Show()

				end

			end

		end

		settings.partypet = nil

	end

end

----------------------

function mod:UpdatePetUnitFrame()

	if ( settings.petunitframe == nil ) then return end

	debug("UpdatePetUnitFrame")

	local PetFrame = _G.PetFrame

	if ( settings.petunitframe and mod:IsEnabled() ) then

		if ( UnitExists("pet") ) then

			PetFrame:Hide()
			PetFrame.Show = dummy

		end

	else

		PetFrame.Show = nil

		if ( UnitExists("pet") ) then

			PetFrame:Show()

		end

		settings.petunitframe = nil

	end

end
