--[[	Player	]]--

local _G = _G
local string = string

----------------------

local GetShapeshiftForm = _G.GetShapeshiftForm
local GetSpecializationInfoByID = _G.GetSpecializationInfoByID
local GetTotemInfo = _G.GetTotemInfo

----------------------

local addonName, ns = ...
local L = ns.L
local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon(addonName)
Player = HideBlizzard:NewModule("Player")
local mod = Player

----------------------

local _, class = UnitClass("player")

local dummy = function() end

local settings

local defaults = {
	profile = {
		["arcanebar"] = nil,
		["chibar"] = nil,
		["combobar"] = nil,
		["insanitybar"] = nil,
		["playercastbar"] = nil,
		["playerunitframe"] = nil,
		["powerbar"] = nil,
		["priestbar"] = nil,
		["runeframe"] = nil,
		["shardframe"] = nil,
		["staggerbar"] = nil,
		["totemframe"] = nil,
	},
}

----------------------

local function out(msg)

	DEFAULT_CHAT_FRAME:AddMessage("|cffAEAEE0"..addonName..":|r "..msg)

end

local function debug(msg)

	local debug = false

	if ( debug ) then

		out(msg)

	end

end

----------------------

function mod:getOptions()

debug("Player getOptions")

return {
	name = L["Player"],
	type = "group",
	arg = "Player",
	get = function(info)

		local key = info[#info]
		local v = settings[key]

		return v

	end,
	set = function(info, value)

		local key = info[#info]

		settings[key] = value

	end,
	args = {
		enabled = {
			name = "|cffFFFF55"..L["Enable Player"].."|r",
			desc = "|cffFFFF55"..L["Hides player related frames."].."|r",
			descStyle = "inline",
			type = "toggle",
			order = 1,
			width = "full",
			get = function() return HideBlizzard:GetModuleEnabled("Player") end,
			set = function(info, value) HideBlizzard:SetModuleEnabled("Player", value) end,
		},
		spacer = {
			name = "",
			type = "description",
			order = 1.5,
		},
		arcanebar = {
			name = L["Arcane Bar"],
			desc = string.format(L["Hides the %smage class%s arcane charges bar."], "|cff69CCF0", "|r"),
			type = "toggle",
			order = 2,
			disabled = function() return ( not Player:IsEnabled()  or class ~= "MAGE" ) end,
			set = function(info, v)

				settings.arcanebar = v

				mod:UpdateArcaneBar()

			end,
		},
		chibar = {
			name = L["Chi Bar"],
			desc = string.format(L["Hides the %smonk class%s chi bar."], "|cff00FF96", "|r"),
			type = "toggle",
			order = 3,
			disabled = function() return ( not Player:IsEnabled()  or class ~= "MONK" ) end,
			set = function(info, v)

				settings.chibar = v

				mod:UpdateChiBar()

			end,
		},
		combobar = {
			name = L["Combo Bar"],
			desc = string.format(L["Hides the %sdruid%s / %srogue%s class combo bar."], "|cffFF7D0A", "|r", "|cffFFF569", "|r"),
			type = "toggle",
			order = 4,
			disabled = function() return not Player:IsEnabled() end,
			set = function(info, v)

				settings.combobar = v

				mod:UpdateComboBar()

			end,
		},
		insanitybar = {
			name = L["Insanity Bar"],
			desc = L["Hides the priest class insanity bar."],
			type = "toggle",
			order = 5,
			disabled = function() return ( not Player:IsEnabled() or class ~= "PRIEST" ) end,
			set = function(info, v)

				settings.insanitybar = v

				mod:UpdateInsanityBar()

			end,
		},
		playercastbar = {
			name = L["Player Cast Bar"],
			desc = L["Hides the player cast bar."],
			type = "toggle",
			order = 6,
			disabled = function() return not Player:IsEnabled() end,
			set = function(info, v)

				settings.playercastbar = v

				mod:UpdatePlayerCastBar()

			end,
		},
		playerunitframe = {
			name = L["Player Unit Frame"],
			desc = L["Hides the player unit frame."],
			type = "toggle",
			order = 7,
			disabled = function() return not Player:IsEnabled() end,
			set = function(info, v)

				settings.playerunitframe = v

				mod:UpdatePlayerUnitFrame()

			end,
		},
		powerbar = {
			name = L["Power Bar"],
			desc = string.format(L["Hides the %spaladin class%s power bar."], "|cffF58CBA", "|r"),
			type = "toggle",
			order = 8,
			disabled = function() return ( not Player:IsEnabled() or class ~= "PALADIN" ) end,
			set = function(info, v)

				settings.powerbar = v

				mod:UpdatePowerBar()

			end,
		},
		priestbar = {
			name = L["Priest Bar"],
			desc = L["Hides the priest class bar."],
			type = "toggle",
			order = 9,
			disabled = function() return ( not Player:IsEnabled() or class ~= "PRIEST" ) end,
			set = function(info, v)

				settings.priestbar = v

				mod:UpdatePriestBar()

			end,
		},
		runeframe = {
			name = L["Rune Frame"],
			desc = string.format(L["Hides the %sdeathknight class%s rune frame."], "|cffC41F3B", "|r"),
			type = "toggle",
			order = 10,
			disabled = function() return ( not Player:IsEnabled() or class ~= "DEATHKNIGHT" ) end,
			set = function(info, v)

				settings.runeframe = v

				mod:UpdateRuneFrame()

			end,
		},
		shardframe = {
			name = L["Shard Frame"],
			desc = string.format(L["Hides the %swarlock class%s shard frame."], "|cff9482C9", "|r"),
			type = "toggle",
			order = 11,
			disabled = function() return ( not Player:IsEnabled() or class ~= "WARLOCK" ) end,
			set = function(info, v)

				settings.shardframe = v

				mod:UpdateShardFrame()

			end,
		},
		staggerbar = {
			name = L["Stagger Bar"],
			desc = string.format(L["Hides the %smonk class%s stagger bar."], "|cff00FF96", "|r"),
			type = "toggle",
			order = 12,
			disabled = function() return ( not Player:IsEnabled() or class ~= "MONK" ) end,
			set = function(info, v)

				settings.staggerbar = v

				mod:UpdateStaggerBar()

			end,
		},
		totemframe = {
			name = L["Totem Frame"],
			desc = L["Hides the totem frame."],
			type = "toggle",
			order = 13,
			disabled = function() return not Player:IsEnabled() end,
			set = function(info, v)

				settings.totemframe = v

				mod:UpdateTotemFrame()

			end,
		},
	},
}
end

----------------------

function mod:OnInitialize()

	debug("Player OnInitialize")

	mod:SetEnabledState(HideBlizzard:GetModuleEnabled("Player"))
	
	mod.db = HideBlizzard.db:RegisterNamespace("Player", defaults)
	
	settings = mod.db.profile

	local options = mod:getOptions()

	HideBlizzard:RegisterModuleOptions("Player", options, "Player")

end

----------------------

function mod:OnEnable()

	debug("Player OnEnable")

	mod:UpdateView()

end

----------------------

function mod:OnDisable()

	debug("Player OnDisable")

	mod:UpdateView()

end

----------------------

function mod:UpdateView()

	debug("ActionBar UpdateView")

	mod:UpdateArcaneBar()
	mod:UpdateChiBar()
	mod:UpdateComboBar()
	mod:UpdateInsanityBar()
	mod:UpdatePlayerCastBar()
	mod:UpdatePlayerUnitFrame()
	mod:UpdatePowerBar()
	mod:UpdatePriestBar()
	mod:UpdateRuneFrame()
	mod:UpdateShardFrame()
	mod:UpdateStaggerBar()
	mod:UpdateTotemFrame()

end

----------------------

function mod:UpdateArcaneBar()

	if ( settings.arcanebar == nil ) then return end

	debug("UpdateArcaneBar")

	local MageArcaneChargesFrame = _G.MageArcaneChargesFrame

	if ( settings.arcanebar and mod:IsEnabled() ) then

		if ( class == "MAGE" and GetSpecializationInfoByID(62) ) then

			MageArcaneChargesFrame:Hide()
			MageArcaneChargesFrame.Show = dummy

		end

	else

		if ( class == "MAGE" and GetSpecializationInfoByID(62) ) then

			MageArcaneChargesFrame.Show = nil
			MageArcaneChargesFrame:Show()

		end

		settings.arcanebar = nil

	end

end

function mod:UpdateChiBar()

	if ( settings.chibar == nil ) then return end

	debug("UpdateChiBar")

	local MonkHarmonyBarFrame = _G.MonkHarmonyBarFrame

	if ( settings.chibar and mod:IsEnabled() ) then

		if ( class == "MONK" and GetSpecializationInfoByID(269) ) then

			MonkHarmonyBarFrame:Hide()
			MonkHarmonyBarFrame.Show = dummy

		end

	else

		if ( class == "MONK" and GetSpecializationInfoByID(269) )then

			MonkHarmonyBarFrame.Show = nil
			MonkHarmonyBarFrame:Show()

		end

		settings.chibar = nil

	end

end

----------------------

function mod:UpdateComboBar()

	if ( settings.combobar == nil ) then return end

	debug("UpdateComboBar")

	local ComboPointPlayerFrame = _G.ComboPointPlayerFrame

	if ( settings.combobar and mod:IsEnabled() ) then

		if ( class == "ROGUE" ) then

			ComboPointPlayerFrame:Hide()
			ComboPointPlayerFrame.Show = dummy

		elseif ( class == "DRUID" ) then

			local catform = GetShapeshiftForm(2)

			if ( catform == 2 ) then
		
				ComboPointPlayerFrame:Hide()
				ComboPointPlayerFrame.Show = dummy

			end

		end

	else

		if ( class == "ROGUE" ) then

			ComboPointPlayerFrame.Show = nil
			ComboPointPlayerFrame:Show()

		elseif ( class == "DRUID" ) then

			local catform = GetShapeshiftForm(2)

			if ( catform == 2 ) then
		
				ComboPointPlayerFrame.Show = nil
				ComboPointPlayerFrame:Show()

			else

				ComboPointPlayerFrame.Show = nil

			end

		end

		settings.combobar = nil

	end

end

----------------------

function mod:UpdateInsanityBar()

	if ( settings.insanitybar == nil ) then return end

	debug("UpdateInsanityBar")

	local InsanityBarFrame = _G.InsanityBarFrame

	if ( settings.insanitybar and mod:IsEnabled() ) then

		if ( class == "PRIEST" and GetSpecializationInfoByID(258) ) then

			InsanityBarFrame:Hide()
			InsanityBarFrame.Show = dummy

		end

	else

		if ( class == "PRIEST" and GetSpecializationInfoByID(258) ) then

			InsanityBarFrame.Show = nil
			InsanityBarFrame:Show()

		end

		settings.insanitybar = nil

	end

end

function mod:UpdatePlayerCastBar()

	if ( settings.playercastbar == nil ) then return end

	debug("UpdatePlayerCastBar")

	local CastingBarFrame = _G.CastingBarFrame

	if ( settings.playercastbar and mod:IsEnabled() ) then

		CastingBarFrame:Hide()
		CastingBarFrame.Show = dummy

	else

		CastingBarFrame.Show = nil

		settings.playercastbar = nil

	end

end

----------------------

function mod:UpdatePlayerUnitFrame()

	if ( settings.playerunitframe == nil ) then return end

	debug("UpdatePlayerUnitFrame")

	local PlayerFrame = _G.PlayerFrame

	if ( settings.playerunitframe and mod:IsEnabled() ) then

		PlayerFrame:Hide()
		PlayerFrame.Show = dummy

	else

		PlayerFrame.Show = nil
		PlayerFrame:Show()

		settings.playerunitframe = nil

	end

end

----------------------

function mod:UpdatePowerBar()

	if ( settings.powerbar == nil ) then return end

	debug("UpdatePowerBar")

	local PaladinPowerBarFrame = _G.PaladinPowerBarFrame

	if ( settings.powerbar and mod:IsEnabled() ) then

		if ( class == "PALADIN" and GetSpecializationInfoByID(70) ) then

			PaladinPowerBarFrame:Hide()
			PaladinPowerBarFrame.Show = dummy

		end

	else

		if ( class == "PALADIN" and GetSpecializationInfoByID(70) ) then

			PaladinPowerBarFrame.Show = nil
			PaladinPowerBarFrame:Show()

		end

		settings.powerbar = nil

	end

end

----------------------

function mod:UpdatePriestBar()

	if ( settings.priestbar == nil ) then return end

	debug("UpdatePriestBar")

	local PriestBarFrame = _G.PriestBarFrame

	if ( settings.priestbar and mod:IsEnabled() ) then

		if ( class == "PRIEST" ) then

			PriestBarFrame:Hide()
			PriestBarFrame.Show = dummy

		end

	else

		if ( class == "PRIEST" ) then

			PriestBarFrame.Show = nil
			PriestBar_Update()

		end

		settings.priestbar = nil

	end

end

function mod:UpdateRuneFrame()

	if ( settings.runeframe == nil ) then return end

	debug("UpdateRuneFrame")

	local RuneFrame = _G.RuneFrame

	if ( settings.runeframe and mod:IsEnabled() ) then

		if ( class == "DEATHKNIGHT" ) then

			RuneFrame:Hide()
			RuneFrame.Show = dummy

		end

	else

		if ( class == "DEATHKNIGHT" ) then

			RuneFrame.Show = nil
			RuneFrame:Show()

		end

		settings.runeframe = nil

	end

end

----------------------

function mod:UpdateShardFrame()

	if ( settings.shardframe == nil ) then return end

	debug("UpdateShardFrame")

	local WarlockPowerFrame = _G.WarlockPowerFrame

	if ( settings.shardframe and mod:IsEnabled() ) then

		if ( class == "WARLOCK" ) then

			WarlockPowerFrame:Hide()
			WarlockPowerFrame.Show = dummy

		end

	else

		if ( class == "WARLOCK" ) then

			WarlockPowerFrame.Show = nil
			WarlockPowerFrame:Show()

		end

		settings.shardframe = nil

	end

end

----------------------

function mod:UpdateStaggerBar()

	if ( settings.staggerbar == nil ) then return end

	debug("UpdateStaggerBar")

	local MonkStaggerBar = _G.MonkStaggerBar

	if ( settings.staggerbar and mod:IsEnabled() ) then

		if ( class == "MONK" and GetSpecializationInfoByID(268) ) then

			MonkStaggerBar:Hide()
			MonkStaggerBar.Show = dummy

		end

	else

		if ( class == "MONK" and GetSpecializationInfoByID(268) ) then

			MonkStaggerBar.Show = nil
			MonkStaggerBar:Show()

		end

		settings.staggerbar = nil

	end

end

function mod:UpdateTotemFrame()

	if ( settings.totemframe == nil ) then return end

	debug("UpdateTotemFrame")

	local TotemFrame = _G.TotemFrame

	if ( settings.totemframe and mod:IsEnabled() ) then

		for i=1, MAX_TOTEMS do

			local haveTotem = GetTotemInfo(i)

			if ( haveTotem ) then
			
				TotemFrame:Hide()
				TotemFrame.Show = dummy

			end

		end

	else

		for i=1, MAX_TOTEMS do

			local haveTotem = GetTotemInfo(i)

			if ( haveTotem ) then
			
				TotemFrame.Show = nil
				TotemFrame:Show()

			end

		end

		settings.totemframe = nil

	end

end
