--[[	Target	]]--

local _G = _G

----------------------

local UnitExists = _G.UnitExists
local UnitClass = _G.UnitClass

----------------------

local addonName, ns = ...
local L = ns.L
local HideBlizzard = LibStub("AceAddon-3.0"):GetAddon(addonName)
Target = HideBlizzard:NewModule("Target")
local mod = Target

----------------------

local _, class = UnitClass("player")

local dummy = function() end

local settings

local defaults = {
	profile = {
		["focusframe"] = nil,
		["targetunitframe"] = nil,
	},
}

----------------------

local function out(msg)

	DEFAULT_CHAT_FRAME:AddMessage("|cffAEAEE0"..addonName..":|r "..msg)

end

local function debug(msg)

	local debug = false

	if ( debug ) then

		out(msg)

	end

end

----------------------

function mod:getOptions()

debug("Target getOptions")

return {
	name = L["Target"],
	type = "group",
	arg = "Target",
	get = function(info)

		local key = info[#info]
		local v = settings[key]

		return v

	end,
	set = function(info, value)

		local key = info[#info]

		settings[key] = value

	end,
	args = {
		enabled = {
			name = "|cffFFFF55"..L["Enable Target"].."|r",
			desc = "|cffFFFF55"..L["Hides target related frames."].."|r",
			descStyle = "inline",
			type = "toggle",
			order = 1,
			width = "full",
			get = function() return HideBlizzard:GetModuleEnabled("Target") end,
			set = function(info, value) HideBlizzard:SetModuleEnabled("Target", value) end,
		},
		spacer = {
			name = "",
			type = "description",
			order = 1.5,
		},
		focusframe = {
			name = L["Focus Frame"],
			desc = L["Hides the focus frame."],
			type = "toggle",
			order = 2,
			disabled = function() return not Target:IsEnabled() end,
			set = function(info, v)

				settings.focusframe = v

				mod:UpdateFocusFrame()

			end,
		},
		targetunitframe = {
			name = L["Target Unit Frame"],
			desc = L["Hides the target unit frame."],
			type = "toggle",
			order = 3,
			disabled = function() return not Target:IsEnabled() end,
			set = function(info, v)

				settings.targetunitframe = v

				mod:UpdateTargetUnitFrame()

			end,
		},
	},
}
end

----------------------

function mod:OnInitialize()

	debug("Target OnInitialize")

	mod:SetEnabledState(HideBlizzard:GetModuleEnabled("Target"))
	
	mod.db = HideBlizzard.db:RegisterNamespace("Target", defaults)
	
	settings = mod.db.profile

	local options = mod:getOptions()

	HideBlizzard:RegisterModuleOptions("Target", options, "Target")

end

----------------------

function mod:OnEnable()

	debug("Target OnEnable")

	mod:UpdateView()

end

----------------------

function mod:OnDisable()

	debug("Target OnDisable")

	mod:UpdateView()

end

----------------------

function mod:UpdateView()

	debug("Target UpdateView")

	mod:UpdateFocusFrame()
	mod:UpdateTargetUnitFrame()

end

----------------------

function mod:UpdateFocusFrame()

	if ( settings.focusframe == nil ) then return end

	debug("UpdateFocusFrame")

	local FocusFrame = _G.FocusFrame

	if ( settings.focusframe and mod:IsEnabled() ) then

		FocusFrame:Hide()
		FocusFrame.Show = dummy

	else

		FocusFrame.Show = nil

		if ( UnitExists("focus") ) then

			FocusFrame:Show()

		end

		settings.focusframe = nil

	end

end

----------------------

function mod:UpdateTargetUnitFrame()

	if ( settings.targetunitframe == nil ) then return end

	debug("UpdateTargetUnitFrame")

	local TargetFrame = _G.TargetFrame

	if ( settings.targetunitframe and mod:IsEnabled() ) then

		TargetFrame:Hide()
		TargetFrame.Show = dummy

	else

		TargetFrame.Show = nil

		if ( UnitExists("target") ) then

			TargetFrame:Show()

		end

		settings.targetunitframe = nil

	end

end
